import { EntityFilter, QueryResult } from '@yukawa/chain-base-angular-domain';
import { Observable } from 'rxjs';
import { Entity } from './entity';
import { EntityCache } from './entity-cache';
import { HttpOptions, RestEntityService } from './rest-entity.service';
/**
 * The CachedEntityService provides wraps the EntityService and provides a cache that stores
 * previously fetched entity objects from the server. Objects within the cache are updated when
 * new values are fetched from the server.
 */
export declare abstract class CachedRestEntityService<T extends Entity> extends RestEntityService<T> {
    readonly cache: EntityCache<T>;
    /**
     * This function is used to map the path ids to a unique key to locate the associated entity
     * within the cache.
     *
     * @param pathIds the pathIds used to determine the key
     * @returns       a unique key to identify the associated entity
     */
    private static keyFromPathIds;
    /**
     * Make an update request to the endpoint, using the supplied object to identify which id to update.
     * If updated, the cache is updated ot set with the entity.
     *
     * @param pathIds An object with keys which match the placeholders within the endpointFormat string.
     * @param obj
     * @param options Optional http options
     */
    update(pathIds: object | T, obj?: T, options?: HttpOptions): Observable<T>;
    /**
     * Make a query request (get all) to the end point, using the supplied parameters to determine path.
     * Caches all returned entities
     *
     * @param pathIds An object with keys which match the placeholders within the endpointFormat string.
     * @param other   Any other data that is needed to be passed to the creation of entities
     *                resulting from this get request.
     * @param options Optional http options
     * @returns a new cold observable
     */
    query<E = T>(pathIds?: object, other?: object, options?: HttpOptions): Observable<QueryResult<E>>;
    query(url: string, filter?: EntityFilter, options?: HttpOptions): Observable<QueryResult<T>>;
    /**
     * First, tries to retrieve from cache, the object with the id, or id field from the pathIds.
     * If found, return the item from cache, otherwise make a get request to the end point,
     * using the supplied parameters to determine path. Caches the returned object
     *
     * @param pathIds Either the id, if a number and maps simple to ':id', otherwise an object
     *                with keys the match the placeholders within the endpointFormat string.
     * @param other   Any other data that is needed to be passed to the creation of entities
     *                resulting from this get request.
     * @param options Optional http options
     */
    fetch(pathIds: number | string | Entity | object, other?: any, options?: HttpOptions): Observable<T>;
    /**
     * First, tries to retrieve from cache, the object with the id, or id field from the pathIds.
     * If found, return the item from cache, otherwise make a get request to the end point,
     * using the supplied parameters to determine path. Caches the returned object
     *
     * @param pathIds Either the id, if a number and maps simple to ':id', otherwise an object
     *                with keys the match the placeholders within the endpointFormat string.
     * @param other   Any other data that is needed to be passed to the creation of entities
     *                resulting from this get request.
     * @param options Optional http options
     */
    get(pathIds: number | string | object, other?: any, options?: HttpOptions): Observable<T>;
    /**
     * Make a create request to the endpoint, using the supplied parameters to determine the path.
     * The results of the request are cached using the key of the entity.
     *
     * @param pathIds An object with keys which match the placeholders within the endpointFormat string.
     * @param data    A FormData or object with the values to pass up in the body of the update/put request.
     * @param other   Any other data needed to be passed to the entity on creation
     * @param options Optional http options
     * @returns a new cold observable with the newly created @type {T}
     */
    create(pathIds?: object, data?: FormData | object, other?: any, options?: HttpOptions): Observable<T>;
    /**
     * Make a delete request to the end point, using the supplied parameters to determine path.
     * If deleted, the object is removed from the cache.
     *
     * @param pathIds Either the id, if a number and maps simple to ':id', otherwise an object
     *                with keys the match the placeholders within the endpointFormat string.
     * @param options Optional http options
     */
    delete(pathIds: number | object, options?: HttpOptions): Observable<object>;
}
